############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.7.4
#
# Michael Würtenberger
# (c) 2019
#
# Licence APL2.0
#
###########################################################
# standard libraries
# external packages
# local import


class Power(object):
    """
    the main window class handles the main menu as well as the show and no show part of
    any other window. all necessary processing for functions of that gui will be linked
    to this class. therefore window classes will have a threadpool for managing async
    processing if needed.
    """

    def __init__(self):

        self.controls = {'POWER_CONTROL_1': self.ui.powerPort1,
                         'POWER_CONTROL_2': self.ui.powerPort2,
                         'POWER_CONTROL_3': self.ui.powerPort3,
                         'POWER_CONTROL_4': self.ui.powerPort4,
                         'POWER_PORT_1': self.ui.powerBootPort1,
                         'POWER_PORT_2': self.ui.powerBootPort2,
                         'POWER_PORT_3': self.ui.powerBootPort3,
                         'POWER_PORT_4': self.ui.powerBootPort4,
                         }

        signals = self.app.power.client.signals
        signals.newNumber.connect(self.updatePowerGui)
        signals.deviceDisconnected.connect(self.clearPowerGui)
        signals.newNumber.connect(self.setPowerNumber)
        signals.defNumber.connect(self.setPowerNumber)
        signals.newSwitch.connect(self.setPowerSwitch)
        signals.defSwitch.connect(self.setPowerSwitch)
        signals.newText.connect(self.setPowerText)
        signals.defText.connect(self.setPowerText)

        self.ui.dewA.valueChanged.connect(self.sendDewA)
        self.ui.dewB.valueChanged.connect(self.sendDewB)

        self.ui.powerPort1.clicked.connect(self.sendPowerPort1)
        self.ui.powerPort2.clicked.connect(self.sendPowerPort2)
        self.ui.powerPort3.clicked.connect(self.sendPowerPort3)
        self.ui.powerPort4.clicked.connect(self.sendPowerPort4)
        self.ui.powerBootPort1.clicked.connect(self.sendPowerBootPort1)
        self.ui.powerBootPort2.clicked.connect(self.sendPowerBootPort2)
        self.ui.powerBootPort3.clicked.connect(self.sendPowerBootPort3)
        self.ui.powerBootPort4.clicked.connect(self.sendPowerBootPort4)
        self.ui.hubUSB.clicked.connect(self.sendHubUSB)
        self.ui.autoDew.clicked.connect(self.sendAutoDew)

    def initConfig(self):
        # config = self.app.config['mainW']
        return True

    def storeConfig(self):
        # config = self.app.config['mainW']
        return True

    def setupIcons(self):
        """
        setupIcons add icon from standard library to certain buttons for improving the
        gui of the app.

        :return:    True if success for test
        """
        return True

    def clearPowerGui(self):
        """
        clearPowerGui changes the state of the Pegasus values to '-'

        :return: success for test
        """

        self.ui.powerTemp.setText('-')
        self.ui.powerHumidity.setText('-')
        self.ui.powerDewPoint.setText('-')
        self.ui.powerCurrent1.setText('-')
        self.ui.powerCurrent2.setText('-')
        self.ui.powerCurrent3.setText('-')
        self.ui.powerCurrent4.setText('-')
        self.ui.consumptionAvgAmps.setText('-')
        self.ui.consumptionAmpHours.setText('-')
        self.ui.consumptionWattHours.setText('-')
        self.ui.sensorVoltage.setText('-')
        self.ui.sensorCurrent.setText('-')
        self.ui.sensorPower.setText('-')
        self.ui.dewCurrentA.setText('-')
        self.ui.dewCurrentB.setText('-')

        return True

    def updatePowerGui(self):
        """
        updatePowerGui changes the style of the button related to the state of the Pegasus

        :return: success for test
        """

        value = self.app.power.data.get('WEATHER_TEMPERATURE', 0)
        self.ui.powerTemp.setText('{0:4.1f}'.format(value))
        value = self.app.power.data.get('WEATHER_HUMIDITY', 0)
        self.ui.powerHumidity.setText('{0:3.0f}'.format(value))
        value = self.app.power.data.get('WEATHER_DEWPOINT', 0)
        self.ui.powerDewPoint.setText('{0:4.1f}'.format(value))

        value = self.app.power.data.get('POWER_CURRENT_1', 0)
        self.ui.powerCurrent1.setText('{0:4.2f}'.format(value))
        value = self.app.power.data.get('POWER_CURRENT_2', 0)
        self.ui.powerCurrent2.setText('{0:4.2f}'.format(value))
        value = self.app.power.data.get('POWER_CURRENT_3', 0)
        self.ui.powerCurrent3.setText('{0:4.2f}'.format(value))
        value = self.app.power.data.get('POWER_CURRENT_4', 0)
        self.ui.powerCurrent4.setText('{0:4.2f}'.format(value))

        value = self.app.power.data.get('CONSUMPTION_AVG_AMPS', 0)
        self.ui.consumptionAvgAmps.setText('{0:4.2f}'.format(value))
        value = self.app.power.data.get('CONSUMPTION_AMP_HOURS', 0)
        self.ui.consumptionAmpHours.setText('{0:4.2f}'.format(value))
        value = self.app.power.data.get('CONSUMPTION_WATT_HOURS', 0)
        self.ui.consumptionWattHours.setText('{0:4.2f}'.format(value))

        value = self.app.power.data.get('SENSOR_VOLTAGE', 0)
        self.ui.sensorVoltage.setText('{0:4.1f}'.format(value))
        value = self.app.power.data.get('SENSOR_CURRENT', 0)
        self.ui.sensorCurrent.setText('{0:4.2f}'.format(value))
        value = self.app.power.data.get('SENSOR_POWER', 0)
        self.ui.sensorPower.setText('{0:4.2f}'.format(value))

        value = self.app.power.data.get('DEW_CURRENT_A', 0)
        self.ui.dewCurrentA.setText('{0:4.2f}'.format(value))
        value = self.app.power.data.get('DEW_CURRENT_B', 0)
        self.ui.dewCurrentB.setText('{0:4.2f}'.format(value))

        return True

    def setPowerNumber(self, deviceName, propertyName):
        """

        :param deviceName:
        :param propertyName:
        :return:
        """

        device = self.app.power.device
        name = self.app.power.name

        if device is None:
            return False
        if deviceName != name:
            return False
        if not getattr(device, 'getNumber', False):
            return False

        for element, value in device.getNumber(propertyName).items():
            if element == 'DEW_A':
                self.ui.dewA.valueChanged.disconnect(self.sendDewA)
                self.ui.dewA.setValue(round(value, -1))
                self.ui.dewA.valueChanged.connect(self.sendDewA)
            elif element == 'DEW_B':
                self.ui.dewB.valueChanged.disconnect(self.sendDewB)
                self.ui.dewB.setValue(round(value, -1))
                self.ui.dewB.valueChanged.connect(self.sendDewB)
            # print(deviceName, propertyName, element, value)

        return True

    def setPowerSwitch(self, deviceName, propertyName):
        """

        :param deviceName:
        :param propertyName:
        :return:
        """

        device = self.app.power.device
        name = self.app.power.name

        if device is None:
            return False
        if deviceName != name:
            return False
        if not getattr(device, 'getSwitch', False):
            return False

        for element, value in device.getSwitch(propertyName).items():
            if element in self.controls:
                self.controls[element].setChecked(value)
            elif propertyName == 'USB_PORT_CONTROL' and element == 'ENABLED':
                self.ui.hubUSB.setChecked(value)
            elif propertyName == 'AUTO_DEW' and element == 'AUTO_DEW_ENABLED':
                self.ui.autoDew.setChecked(value)
            # print(deviceName, propertyName, element, value)

        return True

    def setPowerText(self, deviceName, propertyName):
        """

        :param deviceName:
        :param propertyName:
        :return:
        """

        device = self.app.power.device
        name = self.app.power.name

        if device is None:
            return False
        if deviceName != name:
            return False
        if not getattr(device, 'getText', False):
            return False

        for element, value in device.getText(propertyName).items():
            if element == 'POWER_LABEL_1':
                self.ui.powerLabel1.setText(value)
            elif element == 'POWER_LABEL_2':
                self.ui.powerLabel2.setText(value)
            elif element == 'POWER_LABEL_3':
                self.ui.powerLabel3.setText(value)
            elif element == 'POWER_LABEL_4':
                self.ui.powerLabel4.setText(value)
            # print(deviceName, propertyName, element, value)

        return True

    def sendDewA(self):
        """

        :return: true fot test purpose
        """

        device = self.app.power.device
        name = self.app.power.name
        client = self.app.power.client

        if device is None:
            return False

        dew = device.getNumber('DEW_PWM')
        dew['DEW_A'] = self.ui.dewA.value()
        client.sendNewNumber(deviceName=name,
                             propertyName='DEW_PWM',
                             elements=dew,
                             )
        return True

    def sendDewB(self):
        """

        :return: true fot test purpose
        """

        device = self.app.power.device
        name = self.app.power.name
        client = self.app.power.client

        if device is None:
            return False

        dew = device.getNumber('DEW_PWM')
        dew['DEW_B'] = self.ui.dewB.value()
        client.sendNewNumber(deviceName=name,
                             propertyName='DEW_PWM',
                             elements=dew,
                             )
        return True

    def sendPowerPort1(self):
        """

        :return: true fot test purpose
        """

        device = self.app.power.device
        name = self.app.power.name
        client = self.app.power.client

        if device is None:
            return False

        power = device.getSwitch('POWER_CONTROL')
        power['POWER_CONTROL_1'] = self.ui.powerPort1.isChecked()
        client.sendNewSwitch(deviceName=name,
                             propertyName='POWER_CONTROL',
                             elements=power,
                             )
        return True

    def sendPowerPort2(self):
        """

        :return: true fot test purpose
        """

        device = self.app.power.device
        name = self.app.power.name
        client = self.app.power.client

        if device is None:
            return False

        power = device.getSwitch('POWER_CONTROL')
        power['POWER_CONTROL_2'] = self.ui.powerPort2.isChecked()
        client.sendNewSwitch(deviceName=name,
                             propertyName='POWER_CONTROL',
                             elements=power,
                             )
        return True

    def sendPowerPort3(self):
        """

        :return: true fot test purpose
        """

        device = self.app.power.device
        name = self.app.power.name
        client = self.app.power.client

        if device is None:
            return False

        power = device.getSwitch('POWER_CONTROL')
        power['POWER_CONTROL_3'] = self.ui.powerPort3.isChecked()
        client.sendNewSwitch(deviceName=name,
                             propertyName='POWER_CONTROL',
                             elements=power,
                             )
        return True

    def sendPowerPort4(self):
        """

        :return: true fot test purpose
        """

        device = self.app.power.device
        name = self.app.power.name
        client = self.app.power.client

        if device is None:
            return False

        power = device.getSwitch('POWER_CONTROL')
        power['POWER_CONTROL_4'] = self.ui.powerPort4.isChecked()
        client.sendNewSwitch(deviceName=name,
                             propertyName='POWER_CONTROL',
                             elements=power,
                             )
        return True

    def sendPowerBootPort1(self):
        """

        :return: true fot test purpose
        """

        device = self.app.power.device
        name = self.app.power.name
        client = self.app.power.client

        if device is None:
            return False

        power = device.getSwitch('POWER_ON_BOOT')
        power['POWER_PORT_1'] = self.ui.powerBootPort1.isChecked()
        client.sendNewSwitch(deviceName=name,
                             propertyName='POWER_ON_BOOT',
                             elements=power,
                             )
        return True

    def sendPowerBootPort2(self):
        """

        :return: true fot test purpose
        """

        device = self.app.power.device
        name = self.app.power.name
        client = self.app.power.client

        if device is None:
            return False

        power = device.getSwitch('POWER_ON_BOOT')
        power['POWER_PORT_2'] = self.ui.powerBootPort2.isChecked()
        client.sendNewSwitch(deviceName=name,
                             propertyName='POWER_ON_BOOT',
                             elements=power,
                             )
        return True

    def sendPowerBootPort3(self):
        """

        :return: true fot test purpose
        """

        device = self.app.power.device
        name = self.app.power.name
        client = self.app.power.client

        if device is None:
            return False

        power = device.getSwitch('POWER_ON_BOOT')
        power['POWER_PORT_3'] = self.ui.powerBootPort3.isChecked()
        client.sendNewSwitch(deviceName=name,
                             propertyName='POWER_ON_BOOT',
                             elements=power,
                             )
        return True

    def sendPowerBootPort4(self):
        """

        :return: true fot test purpose
        """

        device = self.app.power.device
        name = self.app.power.name
        client = self.app.power.client

        if device is None:
            return False

        power = device.getSwitch('POWER_ON_BOOT')
        power['POWER_PORT_4'] = self.ui.powerBootPort4.isChecked()
        client.sendNewSwitch(deviceName=name,
                             propertyName='POWER_ON_BOOT',
                             elements=power,
                             )
        return True

    def sendHubUSB(self):
        """

        :return: true fot test purpose
        """

        device = self.app.power.device
        name = self.app.power.name
        client = self.app.power.client

        if device is None:
            return False

        usb = device.getSwitch('USB_PORT_CONTROL')
        usb['ENABLED'] = self.ui.hubUSB.isChecked()
        usb['DISABLED'] = not self.ui.hubUSB.isChecked()
        client.sendNewSwitch(deviceName=name,
                             propertyName='USB_PORT_CONTROL',
                             elements=usb,
                             )
        return True

    def sendAutoDew(self):
        """

        :return: true fot test purpose
        """

        device = self.app.power.device
        name = self.app.power.name
        client = self.app.power.client

        if device is None:
            return False

        autoDew = device.getSwitch('AUTO_DEW')
        autoDew['AUTO_DEW_ENABLED'] = self.ui.autoDew.isChecked()
        autoDew['AUTO_DEW_DISABLED'] = not self.ui.autoDew.isChecked()
        client.sendNewSwitch(deviceName=name,
                             propertyName='AUTO_DEW',
                             elements=autoDew,
                             )
        return True



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.7.4
#
# Michael Würtenberger
# (c) 2019
#
# Licence APL2.0
#
###########################################################
# standard libraries
# external packages
# local import


class SettParkPos(object):
    """
    the main window class handles the main menu as well as the show and no show part of
    any other window. all necessary processing for functions of that gui will be linked
    to this class. therefore window classes will have a threadpool for managing async
    processing if needed.
    """

    def __init__(self):
        # define lists for the entries
        self.posButtons = list()
        self.posTexts = list()
        self.posAlt = list()
        self.posAz = list()

        # dynamically generate the widgets
        self.setupParkPosGui()
        for posText in self.posTexts:
            posText.editingFinished.connect(self.updateParkPosButtonText)
        for button in self.posButtons:
            button.clicked.connect(self.slewToParkPos)

    def initConfig(self):
        """
        initConfig read the key out of the configuration dict and stores it to the gui
        elements. if some initialisations have to be proceeded with the loaded persistent
        data, they will be launched as well in this method.

        :return: True for test purpose
        """
        config = self.app.config['mainW']
        for i, textField in enumerate(self.posTexts):
            keyConfig = 'posText{0:1d}'.format(i)
            textField.setText(config.get(keyConfig, 'Park Pos {0:1d}'.format(i)))
        for i, textField in enumerate(self.posAlt):
            keyConfig = 'posAlt{0:1d}'.format(i)
            textField.setText(config.get(keyConfig, '-'))
        for i, textField in enumerate(self.posAz):
            keyConfig = 'posAz{0:1d}'.format(i)
            textField.setText(config.get(keyConfig, '-'))
        self.updateParkPosButtonText()
        return True

    def storeConfig(self):
        """
        storeConfig writes the keys to the configuration dict and stores. if some
        saving has to be proceeded to persistent data, they will be launched as
        well in this method.

        :return: True for test purpose
        """
        config = self.app.config['mainW']
        for i, textField in enumerate(self.posTexts):
            keyConfig = 'posText{0:1d}'.format(i)
            config[keyConfig] = textField.text()
        for i, textField in enumerate(self.posAlt):
            keyConfig = 'posAlt{0:1d}'.format(i)
            config[keyConfig] = textField.text()
        for i, textField in enumerate(self.posAz):
            keyConfig = 'posAz{0:1d}'.format(i)
            config[keyConfig] = textField.text()
        return True

    def setupIcons(self):
        """
        setupIcons add icon from standard library to certain buttons for improving the
        gui of the app.

        :return:    True if success for test
        """
        return True

    def setupParkPosGui(self):
        """
        setupRelayGui handles the modeldata of list for relay handling. to keep many relay in
        order i collect them in the list for list handling afterwards.

        :return: success for test
        """

        # generate the button list and text entry for later use
        for i in range(0, 8):
            self.posButtons.append(eval('self.ui.posButton{0:1d}'.format(i)))
            self.posTexts.append(eval('self.ui.posText{0:1d}'.format(i)))
            self.posAlt.append(eval('self.ui.posAlt{0:1d}'.format(i)))
            self.posAz.append(eval('self.ui.posAz{0:1d}'.format(i)))
        return True

    def updateParkPosButtonText(self):
        """
        updateParkPosButtonText updates the text in the gui button

        :return: true for test purpose
        """

        for button, textField in zip(self.posButtons, self.posTexts):
            button.setText(textField.text())

        return True

    def slewToParkPos(self):
        """
        slewToParkPos takes the configured data from menu and slews the mount to the
        targeted alt az coordinates.

        :return: success
        """

        for button, posText, alt, az in zip(self.posButtons,
                                            self.posTexts,
                                            self.posAlt,
                                            self.posAz,
                                            ):

            if button != self.sender():
                continue

            try:
                altValue = float(alt.text())
                azValue = float(az.text())
                posTextValue = posText.text()
            except Exception as e:
                self.logger.error(f'no usable values in data: error {e}')
                self.app.message.emit('Missing correct entries in settings', 2)
            else:
                suc = self.app.mount.obsSite.slewAltAz(alt_degrees=altValue,
                                                       az_degrees=azValue,
                                                       slewType='normal')
                if suc:
                    self.app.message.emit(f'Slew to [{posTextValue}]', 0)
                else:
                    self.app.message.emit(f'Cannot slew to [{posTextValue}]', 2)
                return suc

        return False



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# Michael Würtenberger
# (c) 2019
#
# Licence APL2.0
#
###########################################################
from setuptools import setup

mw4_version = '0.107'

setup(
    name='mw4',
    version=mw4_version,
    packages=[
        'mw4',
        'mw4.astrometry',
        'mw4.base',
        'mw4.dome',
        'mw4.environment',
        'mw4.gui',
        'mw4.gui.media',
        'mw4.gui.widgets',
        'mw4.gui.mainWmixin',
        'mw4.imaging',
        'mw4.modeldata',
        'mw4.powerswitch',
        'mw4.remote',
    ],
    python_requires='>=3.7.4',
    install_requires=[
        'mountcontrol',
        'indibase',
        'PyQt5==5.13',
        'matplotlib==3.1.1',
        'astropy==3.2.1',
        'requests==2.22.0',
        'requests_toolbelt==0.9.1',
        'numpy==1.17',
        'skyfield==1.10',
    ],
    url='https://github.com/mworion/MountWizzard4',
    license='APL 2.0',
    author='mw',
    author_email='michael@wuertenberger.org',
    description='tooling for a 10micron mount',
    zip_safe=True,
)



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.7.4
#
# Michael Würtenberger
# (c) 2019
#
# Licence APL2.0
#
###########################################################
# standard libraries
# external packages
import PyQt5.QtCore
from PyQt5.QtCore import Qt
import PyQt5.QtWidgets
import numpy as np
# local import
from mw4.gui.media import resources


class SplashScreen(PyQt5.QtCore.QObject):
    """
    Splash screen show an icon with a progress bar and could send messages to the text
    set in the progress bar. Need the app and the icon as parameter

    Part from Maurizio D'Addona <mauritiusdadd@gmail.com> under license APL2.0
    Ported from PyQt4 to PyQt5

    Agreement for License (email from 04.07.2018):
    Hi Michel,
    sure, there is no problem for me. I'm glad you have found it useful.
    Best regards,
    Maurizio
    """

    __all__ = ['SplashScreen',
               'close',
               'setValue',
               'showMessage',
               'finish',
               ]

    def __init__(self, application=None):
        super().__init__()
        self._qapp = application
        self._pxm = PyQt5.QtGui.QPixmap(':/mw4.ico')

        flags = (Qt.WindowStaysOnTopHint | Qt.X11BypassWindowManagerHint)
        self._qss = PyQt5.QtWidgets.QSplashScreen(self._pxm, flags)
        self._msg = ''
        self._maxv = 100.0
        self._minv = 0.0
        self._cval = 0.0
        self._qss.__drawContents__ = self._qss.drawContents
        self._qss.drawContents = self._drawContents
        self._qss.show()
        self._qss.raise_()
        PyQt5.QtWidgets.QApplication.processEvents()

    def close(self):
        self.update()
        self._qss.close()

    def setValue(self, val):
        for i in np.arange(self._cval, val, self._maxv / 5.0):
            self._cval = i
            self.update()

    def showMessage(self, msg):
        self._msg = msg
        self.update()

    def update(self):
        self._qss.update()
        PyQt5.QtWidgets.QApplication.processEvents()

    def _drawContents(self, painter):
        view_port = painter.viewport()
        w = view_port.right()
        h = view_port.bottom()

        painter.setPen(PyQt5.QtGui.QColor(55, 55, 55, 255))
        painter.setBrush(PyQt5.QtGui.QColor(0, 0, 0, 255))
        painter.drawRect(10, h - 64, w - 20, 19)

        redlg = PyQt5.QtGui.QLinearGradient(0, h - 63, 0, h)
        redlg.setColorAt(0.3, PyQt5.QtGui.QColor(8, 36, 48))
        redlg.setColorAt(0, PyQt5.QtGui.QColor(32, 144, 192))

        painter.setPen(PyQt5.QtCore.Qt.NoPen)
        painter.setBrush(redlg)
        painter.drawRect(13, h - 61, (w - 24) * self._cval / self._maxv, 14)

        painter.setPen(PyQt5.QtCore.Qt.white)

        rect = PyQt5.QtCore.QRectF(10, h - 61, w - 20, 15)
        painter.drawText(rect, PyQt5.QtCore.Qt.AlignCenter, str(self._msg))

    def finish(self, qwid):
        self._qss.finish(qwid)



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.7.4
#
# Michael Würtenberger
# (c) 2019
#
# Licence APL2.0
#
###########################################################
# standard libraries
# external packages
import PyQt5.QtCore
import PyQt5.QtWidgets
import PyQt5.uic
# local import


class SettRelay(object):
    """
    the main window class handles the main menu as well as the show and no show part of
    any other window. all necessary processing for functions of that gui will be linked
    to this class. therefore window classes will have a threadpool for managing async
    processing if needed.
    """

    def __init__(self):
        # define lists for the entries
        self.relayDropDowns = [self.ui.relayFun0,
                               self.ui.relayFun1,
                               self.ui.relayFun2,
                               self.ui.relayFun3,
                               self.ui.relayFun4,
                               self.ui.relayFun5,
                               self.ui.relayFun6,
                               self.ui.relayFun7,
                               ]
        self.relayDropDownKeys = ['relay0index',
                                  'relay1index',
                                  'relay2index',
                                  'relay3index',
                                  'relay4index',
                                  'relay5index',
                                  'relay6index',
                                  'relay7index',
                                  ]
        self.relayButtons = [self.ui.relayButton0,
                             self.ui.relayButton1,
                             self.ui.relayButton2,
                             self.ui.relayButton3,
                             self.ui.relayButton4,
                             self.ui.relayButton5,
                             self.ui.relayButton6,
                             self.ui.relayButton7,
                             ]
        self.relayButtonTexts = [self.ui.relayButtonText0,
                                 self.ui.relayButtonText1,
                                 self.ui.relayButtonText2,
                                 self.ui.relayButtonText3,
                                 self.ui.relayButtonText4,
                                 self.ui.relayButtonText5,
                                 self.ui.relayButtonText6,
                                 self.ui.relayButtonText7,
                                 ]
        self.relayButtonTextKeys = ['relay0buttonText',
                                    'relay1buttonText',
                                    'relay2buttonText',
                                    'relay3buttonText',
                                    'relay4buttonText',
                                    'relay5buttonText',
                                    'relay6buttonText',
                                    'relay7buttonText',
                                    ]

        # dynamically generate the widgets
        self.setupRelayGui()

        # make the gui signals linked to slots
        self.ui.relayHost.editingFinished.connect(self.relayHost)
        self.ui.relayUser.editingFinished.connect(self.relayUser)
        self.ui.relayPassword.editingFinished.connect(self.relayPassword)
        for relayButtonText in self.relayButtonTexts:
            relayButtonText.editingFinished.connect(self.updateRelayButtonText)
        for button in self.relayButtons:
            button.clicked.connect(self.relayButtonPressed)

    def initConfig(self):
        """
        initConfig read the key out of the configuration dict and stores it to the gui
        elements. if some initialisations have to be proceeded with the loaded persistent
        data, they will be launched as well in this method.

        :return: True for test purpose
        """
        config = self.app.config['mainW']
        self.ui.relayHost.setText(config.get('relayHost', ''))
        self.relayHost()
        self.ui.relayUser.setText(config.get('relayUser', ''))
        self.relayUser()
        self.ui.relayPassword.setText(config.get('relayPassword', ''))
        self.relayPassword()
        for button, key in zip(self.relayButtonTexts, self.relayButtonTextKeys):
            button.setText(config.get(key, ''))
        for dropDown, key in zip(self.relayDropDowns, self.relayDropDownKeys):
            dropDown.setCurrentIndex(config.get(key, 0))
        self.updateRelayButtonText()
        return True

    def storeConfig(self):
        """
        storeConfig writes the keys to the configuration dict and stores. if some
        saving has to be proceeded to persistent data, they will be launched as
        well in this method.

        :return: True for test purpose
        """
        config = self.app.config['mainW']
        config['relayHost'] = self.ui.relayHost.text()
        config['relayUser'] = self.ui.relayUser.text()
        config['relayPassword'] = self.ui.relayPassword.text()
        for button, key in zip(self.relayButtonTexts, self.relayButtonTextKeys):
            config[key] = button.text()
        for dropDown, key in zip(self.relayDropDowns, self.relayDropDownKeys):
            config[key] = dropDown.currentIndex()
        return True

    def setupIcons(self):
        """
        setupIcons add icon from standard library to certain buttons for improving the
        gui of the app.

        :return:    True if success for test
        """
        return True

    def setupRelayGui(self):
        """
        setupRelayGui handles the modeldata of list for relay handling.

        :return: success for test
        """

        for dropDown in self.relayDropDowns:
            dropDown.clear()
            dropDown.setView(PyQt5.QtWidgets.QListView())
            dropDown.addItem('Switch - Toggle')
            dropDown.addItem('Pulse 0.5 sec')
        return True

    def updateRelayButtonText(self):
        """

        :return:
        """
        for button, textField in zip(self.relayButtons, self.relayButtonTexts):
            button.setText(textField.text())

    def doRelayAction(self, relayIndex=0):
        """

        :param relayIndex: relayIndex according to pressed button
        :return: success
        """

        action = self.relayDropDowns[relayIndex].currentIndex()
        if action == 0:
            suc = self.app.relay.switch(relayIndex)
        elif action == 1:
            suc = self.app.relay.pulse(relayIndex)
        else:
            suc = False
        return suc

    def relayButtonPressed(self):
        """
        relayButtonPressed reads the button and starts the relay action on the box.

        :return: success for test
        """

        suc = False
        for i, button in enumerate(self.relayButtons):
            if button != self.sender():
                continue
            suc = self.doRelayAction(i)
        if not suc:
            self.app.message.emit('Relay action cannot be performed', 2)
            return False
        return True

    def relayHost(self):
        self.app.relay.host = self.ui.relayHost.text()

    def relayUser(self):
        self.app.relay.user = self.ui.relayUser.text()

    def relayPassword(self):
        self.app.relay.password = self.ui.relayPassword.text()
